"""
apartments API library
~~~~~~~~~~~~~~~~~~~~~~

apartments is a fast and expressive apartments.com API wrapper,
written in Python, for human beings.
"""

from .url import URL

__version__ = "0.0.1"
